import re
from typing import IO
import click
import logging


logging.basicConfig(level=logging.ERROR)
text: list[str]
jira_url: str = "https://jira.suse.com/browse/"
bugzilla_url: str = "https://bugzilla.suse.com/show_bug.cgi?id="
gitlab_url: str = "https://gitlab.suse.de/documentation/release-notes-sles/-/commit/"
search_pattern: str = r"((?:jsc#SLE-|bsc#)[0-9]+)"


def format_section_id(id_string: str) -> str:
    newstring: str = ""
    newstring = id_string.replace("#", "-")
    return newstring


def make_adoc_url(url: str, title: str) -> str:
    return "{}[{}]".format(url, title)


def make_jira_url(issue_number: int) -> str:
    return make_adoc_url(jira_url + str(issue_number), "Jira")


def make_bugzilla_url(issue_number: int) -> str:
    return make_adoc_url(bugzilla_url + str(issue_number), "Bugzilla")


def make_entry(section_id: str, parentheses_text: str) -> str:
    return "Added <<{}>> ({})".format(section_id, parentheses_text)


@click.group()
def cli():
    pass


def parse(input_text: str = ""):
    """Parses text for Jira or Bugzilla issue numbers and creates entries from them"""
    if input_text == "":
        logging.error("No text provided for parsing.")
    match input_text:
        case [text] if type(text) == str:
            logging.debug(f"{text} is a string.")
            parse_text(text)
        case [*line]:
            logging.debug("This is a list of lines.")
            for line in input_text:
                line = line.strip()
                parse_text(line)
        case _:
            logging.debug("This is nothing I recognize.")


@cli.command("parse-text")
@click.argument("text")
def parse_text_command(text):
    parse_text(text)


def parse_text(line):
    if results := re.findall(search_pattern, line, re.IGNORECASE):
        logging.debug(f"Found matches: {results}")
        parentheses_items: list = []
        # FIXME: add links to other mentioned sections, for now just link the first one
        section_id = format_section_id(results[0])
        for match in results:
            issue_number = match.split("#").pop()
            if "jsc" in match:
                parentheses_items.append(make_jira_url(issue_number))
            if "bsc" in match:
                parentheses_items.append(make_bugzilla_url(issue_number))
        parentheses_text = ", ".join(parentheses_items)
        click.echo(make_entry(section_id, parentheses_text))
    else:
        logging.debug(f"Nothing found in line '{line}'. Ignoring.")


@cli.command()
@click.argument("filename", type=click.File())
def parse_file(filename: IO):
    parse(filename.readlines())


if __name__ == "__main__":
    cli()
